﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using DG.Tweening;

public class BatteryHandler : MonoBehaviour
{
    public enum BatteryLocation
    {
        Inventory,
        Dock,
        Ferry
    };

	public const float BatteryFull = 1;
	public const float BatteryEmpty = 0;
	public const float ChargeIncrement = 0.01f;

    public BatteryLocation Location;
    public float Charge; // BatteryCharge 0 = empty, 1 = full

    public GameObject ChargeGO;
	public Color ChargeLowColor;
	public Color ChargeHighColor;

	public Renderer RendererCharge;

	Coroutine setChargeVisualsCoroutine;

    public void Reset()
    {
        Location = BatteryLocation.Inventory;
		Charge = BatteryFull;
		SetChargeVisuals();
        
    }

    public void AddCharge(float chargeIncrement)
    {

		Charge = Charge + chargeIncrement;

        if (Charge >= BatteryFull)
		{
			Charge = 1.0f;
		}

		Debug.Log("Added charge - Current charge: " + Charge);
		SetChargeVisuals();
    }

	/// <summary>
	/// Reduces the charge. Duration is mainly used to lerp the visuals instead of snapping to a larger value.
	/// </summary>
	/// <param name="chargeDecrement">Charge decrement.</param>
	/// <param name="duration">Duration. The time it takes for the visuals to lerp to the new position</param>
    public void ReduceCharge(float chargeDecrement, float duration)
    {
		Charge = Charge - chargeDecrement;

        if (Charge <= BatteryEmpty)
		{
			Charge = 0.0f; 
		}
		Debug.Log("Reduced charge - Current charge: " + Charge);


		if(setChargeVisualsCoroutine != null)
			StopCoroutine(setChargeVisualsCoroutine);

		setChargeVisualsCoroutine = StartCoroutine(SetChargeVisuals(duration));
    }

	public void ReduceCharge(float chargeDecrement)
	{
		Charge = Charge - chargeDecrement;

		if (Charge <= BatteryEmpty)
		{
			Charge = 0.0f;
		}
		Debug.Log("Battery Empty - Current charge: " + Charge);

		SetChargeVisuals();
	}
	
	void SetChargeVisuals()
	{
		ChargeGO.transform.localScale = new Vector3(ChargeGO.transform.localScale.x, ChargeGO.transform.localScale.y, Mathf.Lerp(0.0f, 1.0f, Charge/BatteryFull));
		RendererCharge.material.color = Color.Lerp(ChargeLowColor, ChargeHighColor, (Charge/BatteryFull));
	}

	// TODO: merge these two charge visuals. 
	// TODO: Use gradient instead of two blend colors.
	IEnumerator SetChargeVisuals(float duration)
	{
		ChargeGO.transform.DOScaleZ(Mathf.Clamp01(Charge/BatteryFull), duration);
		RendererCharge.material.DOColor(Color.Lerp(ChargeLowColor, ChargeHighColor, Mathf.Clamp01(Charge/BatteryFull)), duration);
		yield return null;
	}


}