﻿using System;
using System.Collections;
using System.Collections.Generic;
using FerryMachine;
using UnityEngine;

namespace ChargingMachine
{
    public class ChargingMachine : MonoBehaviour, IChargingMachine
    {
        private IChargingContext _context;
        private bool _paused;

        public static Dictionary<string, object> Flags = new Dictionary<string, object>();

        public bool Paused
        {
            get { return _paused; }
        }

        public bool IsActive
        {
            get { return _context != null && _context.IsActive; }
        }

        public ChargingStates? State
        {
            get { return _context != null ? _context.State : (ChargingStates?)null; }
        }

        public void Play()
        {
            // Fire up state machine
            _context = new ChargingContext(this);
            _context.StateChange += (sender, args) =>
            {
                Debug.Log(string.Format("New state for '{0}' is '{1}'", GetType().Name, args.NewState));
            };

            StartCoroutine(RunMachine());
        }

        public void Reset()
        {
            _context = null;
        }

        private IEnumerator RunMachine()
        {
            while (IsActive)
            {
                if (!Paused)
                {
                    _context.Process(TimeSpan.FromSeconds(Time.time), Flags);
                    Flags.Clear();
                }

                yield return null;
            }
        }

        public void DoBlackout()
        {
            FerrySceneManager.Instance.PopupHandler.Show(@"Oi! Hele nabolaget gikk i svart. Dagens kraftsystem " +
                                                         "er ikke dimensjonert til at hurtigladere med så stor effekt " +
                                                         "kan koble seg direkte på strømnettet. Du må koble annerledes.",
                Color.black);
        }

        public void DoCharging(float interval)
        {
            FerrySceneManager.Instance.GameHandler.FerryHandler.SetCharging(interval);
			if(interval<0)
			{
				SceneFXHandler.Instance.FadeWeakCurrent(0.0f);
				SceneFXHandler.Instance.FadeStrongCurrent(0.0f);
			}
			if(interval>0)
			{
				SceneFXHandler.Instance.FadeWeakCurrent(1.0f);
				SceneFXHandler.Instance.FadeStrongCurrent(1.0f);
			}
			if(interval>5)
			{
				SceneFXHandler.Instance.FadeWeakCurrent(4.0f);
				SceneFXHandler.Instance.FadeStrongCurrent(4.0f);
			}
        }

        public void DoTimeout()
        {
        }

        public void Close()
        {
        }
    }
}