Shader "Custom/Metaballs" {
	Properties{
		_MainTex("Texture", 2D) = "white" { }


		[Header(Red chanel)]
		_ROuterTreshhold("Outer treshhold", Range(0.0, 1)) = 0.3
			_ROuterColor("Outer Color", Color) = (1.0, 0.0, 0.0, 1.0)
			_RCenterTreshhold("Center treshhold", Range(0.0, 1)) = 0.5
			_RCenterColor("Center Color", Color) = (1.0, 0.0, 0.0, 1.0)
			_RInnerTreshhold("Inner treshhold", Range(0.0, 1)) = 0.8
			_RInnerColor("Inner Color", Color) = (1.0, 0.0, 0.0, 1.0)

			[Header(Green chanel)]
		_GOuterTreshhold("Outer treshhold", Range(0.0, 1)) = 0.3
			_GOuterColor("Outer Color", Color) = (0.0, 1.0, 0.0, 1.0)
			_GCenterTreshhold("Center treshhold", Range(0.0, 1)) = 0.5
			_GCenterColor("Center Color", Color) = (0.0, 1.0, 0.0, 1.0)
			_GInnerTreshhold("Inner treshhold", Range(0.0, 1)) = 0.8
			_GInnerColor("Inner Color", Color) = (0.0, 1.0, 0.0, 1.0)

			[Header(Blue chanel)]
		_BOuterTreshhold("Outer treshhold", Range(0.0, 1)) = 0.3
			_BOuterColor("Outer Color", Color) = (0.0, 0.0, 1.0, 1.0)
			_BCenterTreshhold("Center treshhold", Range(0.0, 1)) = 0.5
			_BCenterColor("Center Color", Color) = (0.0, 0.0, 1.0, 1.0)
			_BInnerTreshhold("Inner treshhold", Range(0.0, 1)) = 0.8
			_BInnerColor("Inner Color", Color) = (0.0, 0.0, 1.0, 1.0)
	}
	/// <summary>
	/// Multiple metaball shader.
	/// 
	/// Separates each particle by color and overrides it with the one specified.
	/// Notice the texture that passes through this shader only looks at particles, and has a black background.
	/// The core element for the color merging is the floor function, try tweaking it to achive the desired result.
	///
	/// Visit: www.codeartist.mx for more stuff. Thanks for checking out this example.
	/// Credit: Rodrigo Fernandez Diaz
	/// Contact: q_layer@hotmail.com
	/// </summary>
	SubShader{
			Tags{ "Queue" = "Transparent" }
			Pass{
				Blend SrcAlpha OneMinusSrcAlpha
				Lighting Off
				Alphatest Greater 0.01
				ZWrite Off
				ZTest Always
				CGPROGRAM
#pragma vertex vert
#pragma fragment frag
#include "UnityCG.cginc"
				float4 _Color;

				float _ROuterTreshhold;
				float4 _ROuterColor;
				float _RCenterTreshhold;
				float4 _RCenterColor;
				float _RInnerTreshhold;
				float4 _RInnerColor;

				float _GOuterTreshhold;
				float4 _GOuterColor;
				float _GCenterTreshhold;
				float4 _GCenterColor;
				float _GInnerTreshhold;
				float4 _GInnerColor;

				float _BOuterTreshhold;
				float4 _BOuterColor;
				float _BCenterTreshhold;
				float4 _BCenterColor;
				float _BInnerTreshhold;
				float4 _BInnerColor;

				sampler2D _MainTex;
				struct v2f {
					float4  pos : SV_POSITION;
					float2  uv : TEXCOORD0;
				};
				float4 _MainTex_ST;
				v2f vert(appdata_base v){
					v2f o;
					o.pos = mul(UNITY_MATRIX_MVP, v.vertex);
					o.uv = TRANSFORM_TEX(v.texcoord, _MainTex);
					return o;
				}

				// Here goes the metaball magic
				float COLOR_TRESHHOLD = 0.2; //To separate and process each color.
				float4 frag(v2f i) : COLOR{
					half4 texcol = tex2D(_MainTex, i.uv);
					float4 finalRColor = half4(0.0, 0.0, 0.0, 0.0);
					if (texcol.r > _ROuterTreshhold){
						if (texcol.r > _RInnerTreshhold)
						{
							finalRColor = _RInnerColor;
						}
						else if (texcol.r > _RCenterTreshhold)
						{
							finalRColor = _RCenterColor;
						}
						else
						{
							finalRColor = _ROuterColor;
						}
					}
					float4 finalGColor = half4(0.0, 0.0, 0.0, 0.0);
						if (texcol.g > _GOuterTreshhold){
							if (texcol.g > _GInnerTreshhold)
							{
								finalGColor = _GInnerColor;
							}
							else if (texcol.g > _GCenterTreshhold)
							{
								finalGColor = _GCenterColor;
							}
							else
							{
								finalGColor = _GOuterColor;
							}
						}
					float4 finalBColor = half4(0.0, 0.0, 0.0, 0.0);
						if (texcol.b > _BOuterTreshhold){
							if (texcol.b > _BInnerTreshhold)
							{
								finalBColor = _BInnerColor;
							}
							else if (texcol.b > _BCenterTreshhold)
							{
								finalBColor = _BCenterColor;
							}
							else
							{
								finalBColor = _BOuterColor;
							}
						}
					float sum = texcol.r + texcol.g + texcol.b;
					
					if (sum == 0) return half4(0.0, 0.0, 0.0, 0.0);

					float4 finalColor = (finalRColor * texcol.r / sum)
						+ (finalGColor * texcol.g / sum)
						+ (finalBColor * texcol.b / sum);


					return finalColor;
					/*
					else if(texcol.g>_GOuterTreshhold){ //This is for the gas effect
					finalColor=half4(texcol.g,texcol.g,texcol.g,0.5f);// All channels white with some transparency
					finalColor=floor(finalColor/0.05)*0.3;  //Blend it in
					finalColor.a=0.5f; // To add some transparency
					}
					else if(texcol.b>_BOuterTreshhold){	//This is for the water effect
					finalColor=half4(0.0,texcol.b/2.0,texcol.b,0.5);

					finalColor.b=floor((finalColor.b/0.1)*0.5);
					}	    */
				}
					ENDCG

			}
		}
		Fallback "VertexLit"
}